@@ -1,6 +1,6 @@
 ;;;; remote-rep.jl -- Remote file access via the rep-remote program
 ;;;  Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
-;;;  $Id: rep.jl,v 1.17 2001/03/26 21:51:12 jsh Exp $
+;;;  $Id$
 
 ;;; This file is part of librep.
 
@@ -226,6 +226,7 @@
   (or (eq (aref session remote-rep-status) 'success)
       (signal 'file-error
 	      (list (aref session remote-rep-error)
+		    type
 		    (format nil "%s@%s %s"
 			    (aref session remote-rep-user)
 			    (aref session remote-rep-host) args)))))
@@ -298,9 +299,9 @@
 			 (substring output point))
 		   (setq point (length output)))))
 	      (t
-	       (unless (string-looking-at "\\s*$" output point)
-		 (format standard-error "remote-rep: unhandled output %S\n"
-			 (substring output point)))
+;	       (unless (string-looking-at "\\s*$" output point)
+;		 (format standard-error "remote-rep: unhandled output %S\n"
+;			 (substring output point)))
 	       (setq point (length output))))))))
 
 (defun remote-rep-sentinel (process)
