@@ -1,6 +1,6 @@
 #| bootstrap for rep.lang.interpreter
 
-   $Id: interpreter.jl,v 1.15 2001/10/12 04:21:20 jsh Exp $
+   $Id$
 
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -296,9 +296,9 @@ unconditionally sets its value to VALUE (or false if VALUE isn't
 defined). If DOC is given it will be installed as the documentation
 string associated with VARIABLE."
 
-  `(progn
-     (defvar ,var nil ,doc)
-     (setq ,var ,value)))
+  (list 'progn
+	(list 'defvar var nil doc)
+	(list 'setq var value)))
 
 (export-bindings '(setq-default define-special-variable))
 
