@@ -1,6 +1,6 @@
 #| modules.jl -- module handling for the compiler
 
-   $Id: modules.jl,v 1.43 2002/06/22 23:12:26 jsh Exp $
+   $Id$
 
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
 
@@ -150,8 +150,9 @@
 	value)))
 
   (defun compiler-boundp (var)
-    (or (locate-variable var)
-	(and (special-variable-p var) (boundp var))))
+    (and (symbolp var)
+	 (or (locate-variable var)
+	     (and (special-variable-p var) (boundp var)))))
 
   ;; return t if the binding of VAR comes from the rep (built-ins) module
   (defun compiler-binding-from-rep-p (var)
