@@ -1,6 +1,6 @@
 /* datums.c -- user-defined opaque types
    Copyright (C) 2000 John Harper <john@dcs.warwick.ac.uk>
-   $Id: datums.c,v 1.7 2000/06/07 18:12:52 john Exp $
+   $Id$
 
    This file is part of librep.
 
@@ -26,6 +26,7 @@
 
 #define _GNU_SOURCE
 
+#define rep_DEFINE_QNIL 1
 #include "repint.h"
 
 static int datum_type;
@@ -39,8 +40,8 @@ static repv printer_alist;
 #define DATUM_ID(x) (rep_TUPLE(x)->a)
 #define DATUM_VALUE(x) (rep_TUPLE(x)->b)
 
-/* support for scheme boolean type */
-repv rep_scm_t, rep_scm_f;
+/* This is what Qnil points to */
+rep_tuple rep_eol_datum;
 
 
 /* type hooks */
@@ -56,18 +57,26 @@ datum_cmp (repv d1, repv d2)
 
 static void
 datum_print (repv stream, repv arg)
-{
-    repv printer = Fassq (DATUM_ID (arg), printer_alist);
-    if (printer && rep_CONSP (printer) && rep_CDR (printer) != Qnil)
-	rep_call_lisp2 (rep_CDR (printer), arg, stream);
-    else if (rep_SYMBOLP (DATUM_ID (arg)))
+{    
+    if (arg == Qnil)
     {
-	rep_stream_puts (stream, "#<datum ", -1, rep_FALSE);
-	rep_stream_puts (stream, rep_PTR (rep_SYM (DATUM_ID (arg))->name), -1, rep_TRUE);
-	rep_stream_putc (stream, '>');
+	DEFSTRING (eol, "()");
+	rep_stream_puts (stream, rep_PTR (rep_VAL (&eol)), 2, rep_TRUE);
     }
     else
-	rep_stream_puts (stream, "#<datum>", -1, rep_FALSE);
+    {
+	repv printer = Fassq (DATUM_ID (arg), printer_alist);
+	if (printer && rep_CONSP (printer) && rep_CDR (printer) != Qnil)
+	    rep_call_lisp2 (rep_CDR (printer), arg, stream);
+	else if (rep_SYMBOLP (DATUM_ID (arg)))
+	{
+	    rep_stream_puts (stream, "#<datum ", -1, rep_FALSE);
+	    rep_stream_puts (stream, rep_PTR (rep_SYM (DATUM_ID (arg))->name), -1, rep_TRUE);
+	    rep_stream_putc (stream, '>');
+	}
+	else
+	    rep_stream_puts (stream, "#<datum>", -1, rep_FALSE);
+    }
 }
 
 
@@ -75,7 +84,7 @@ datum_print (repv stream, repv arg)
 
 DEFUN ("make-datum", Fmake_datum,
        Smake_datum, (repv value, repv id), rep_Subr2) /*
-::doc:make-datum::
+::doc:rep.data.datums#make-datum::
 make-datum VALUE ID
 
 Create and return a new data object of type ID (an arbitrary value), it
@@ -87,7 +96,7 @@ will have object VALUE associated with it.
 
 DEFUN ("define-datum-printer", Fdefine_datum_printer,
        Sdefine_datum_printer, (repv id, repv printer), rep_Subr2) /*
-::doc:define-datum-printer::
+::doc:rep.data.datums#define-datum-printer::
 define-datum-printer ID PRINTER
 
 Register a custom printer for all datums with type ID. When these
@@ -104,7 +113,7 @@ arguments, the datum and the stream to print to.
 }
 
 DEFUN ("datum-ref", Fdatum_ref, Sdatum_ref, (repv obj, repv id), rep_Subr2) /*
-::doc:datum-ref::
+::doc:rep.data.datums#datum-ref::
 datum-ref DATUM ID
 
 If data object DATUM has type ID, return its associated value, else
@@ -117,7 +126,7 @@ signal an error.
 
 DEFUN ("datum-set", Fdatum_set, Sdatum_set,
        (repv obj, repv id, repv value), rep_Subr3) /*
-::doc:datum-set::
+::doc:rep.data.datums#datum-set::
 datum-set DATUM ID VALUE
 
 If data object DATUM has type ID, modify its associated value to be
@@ -131,7 +140,7 @@ VALUE, else signal an error.
 
 DEFUN ("has-type-p", Fhas_type_p,
        Shas_type_p, (repv arg, repv id), rep_Subr2) /*
-::doc:has-type-p::
+::doc:rep.data.datums#has-type-p::
 has-type-p ARG ID
 
 Return `t' if object ARG has data type ID (and thus was initially
@@ -141,27 +150,30 @@ created using the `make-datum' function).
     return (DATUMP (arg) && DATUM_ID (arg) == id) ? Qt : Qnil;
 }
 
-
-/* support for scheme boolean values */
-
-DEFUN ("%scheme-bool-printer", F_scheme_bool_printer,
-       S_scheme_bool_printer, (repv obj, repv stream), rep_Subr2)
-{
-    rep_stream_puts (stream, obj == rep_scm_t ? "#t" : "#f", 2, rep_FALSE);
-    return Qnil;
-}
-
 
 /* dl hooks */
 
 void
-rep_datums_init (void)
+rep_pre_datums_init (void)
 {
     datum_type = rep_register_new_type ("datum", datum_cmp,
 					datum_print, datum_print,
 					0, rep_mark_tuple,
 					0, 0, 0, 0, 0, 0, 0);
 
+    /* Including CELL_MARK_BIT means we don't have to worry about
+       GC; the cell will never get remarked, and it's not on any
+       allocation lists to get swept up from. */
+    rep_eol_datum.car = datum_type | rep_CELL_STATIC_BIT | rep_CELL_MARK_BIT;
+    rep_eol_datum.a = rep_VAL (&rep_eol_datum);
+    rep_eol_datum.b = rep_VAL (&rep_eol_datum);
+}
+
+void
+rep_datums_init (void)
+{
+    repv tem = rep_push_structure ("rep.data.datums");
+
     rep_ADD_SUBR (Smake_datum);
     rep_ADD_SUBR (Sdefine_datum_printer);
     rep_ADD_SUBR (Sdatum_ref);
@@ -170,11 +182,5 @@ rep_datums_init (void)
     printer_alist = Qnil;
     rep_mark_static (&printer_alist);
 
-    rep_ADD_INTERNAL_SUBR (S_scheme_bool_printer);
-    rep_scm_t = Fmake_datum (Qt, rep_VAL (&S_scheme_bool_printer));
-    rep_scm_f = Fmake_datum (Qnil, rep_VAL (&S_scheme_bool_printer));
-    Fdefine_datum_printer (rep_VAL (&S_scheme_bool_printer),
-			   rep_VAL (&S_scheme_bool_printer));
-    rep_mark_static (&rep_scm_t);
-    rep_mark_static (&rep_scm_f);
+    rep_pop_structure (tem);
 }
