@@ -1,6 +1,6 @@
 /* ffi.c -- foreigh function interface plugin
 
-   $Id: ffi.c,v 1.4 2003/11/25 03:20:52 jsh Exp $
+   $Id$
 
    Copyright (C) 2003 John Harper <jsh@unfactored.org>
 
@@ -72,6 +72,9 @@
 
 #ifdef HAVE_FFI_H
 #include <ffi.h>
+#ifndef ALIGN /* was in older ffi.h */
+#define ALIGN(v, a)  (((((unsigned) (v))-1) | ((a)-1))+1)
+#endif
 #endif
 
 #if SIZEOF_VOID_P == SIZEOF_LONG
@@ -150,10 +153,10 @@ static rep_ffi_interface **ffi_interfaces;
 static rep_bool
 ffi_types_equal_p (const rep_ffi_type *a, const rep_ffi_type *b)
 {
-    if (a->type != NULL && a->type == b->type)
-	return rep_TRUE;
     if (a->subtype != b->subtype)
 	return rep_FALSE;
+    if (a->type != NULL && a->type == b->type)
+	return rep_TRUE;
 
     switch (a->subtype)
     {
@@ -307,9 +310,11 @@ rep_ffi_marshal (unsigned int type_id, repv value, char *ptr)
 	    *(double *)ptr = (double) rep_get_float (value);
 	    return ptr + sizeof (double);
 
+#if FFI_TYPE_LONGDOUBLE != FFI_TYPE_DOUBLE
 	case FFI_TYPE_LONGDOUBLE:
 	    *(long double *)ptr = (long double) rep_get_float (value);
 	    return ptr + sizeof (long double);
+#endif
 
 	case FFI_TYPE_UINT8:
 	    *(uint8_t *)ptr = (uint8_t) rep_get_long_int (value);
@@ -344,7 +349,7 @@ rep_ffi_marshal (unsigned int type_id, repv value, char *ptr)
 	    return ptr + sizeof (int64_t);
 
 	case FFI_TYPE_POINTER:
-	    *(void **)ptr = rep_get_pointer (value);
+	    *(void **)ptr = (rep_STRINGP(value)) ? rep_STR (value) : rep_get_pointer (value);
 	    return ptr + sizeof (void *);
 
 	case FFI_TYPE_STRUCT:		/* FIXME: */
@@ -435,9 +440,11 @@ rep_ffi_demarshal (unsigned int type_id, char *ptr, repv *value)
 	    *value = rep_make_float (*(double *)ptr, rep_TRUE);
 	    return ptr + sizeof (double);
 
+#if FFI_TYPE_LONGDOUBLE != FFI_TYPE_DOUBLE
 	case FFI_TYPE_LONGDOUBLE:
 	    *value = rep_make_float (*(long double *)ptr, rep_TRUE);
 	    return ptr + sizeof (long double);
+#endif
 
 	case FFI_TYPE_UINT8:
 	    *value = rep_MAKE_INT (*(uint8_t *)ptr);
@@ -588,7 +595,7 @@ DEFUN ("ffi-type", Fffi_type, Sffi_type,
 {
     rep_ffi_alias *s;
 
-    rep_DECLARE (1, base, rep_VALID_INTERFACE_P (base));
+    rep_DECLARE (1, base, rep_VALID_TYPE_P (base));
 
     s = rep_alloc (sizeof (rep_ffi_alias));
 
