@@ -6,24 +6,19 @@ ltconf=/tmp/libtool.conf.$$
 . $ltconf
 rm -f $ltconf
 
-prefix=$1
-libdir=$2
-version=$3
-LIBS=$4
-repexecdir=$5
+prefix="$1"
+libdir="$2"
+version="$3"
+LIBS="$4"
+repexecdir="$5"
+sitelispdir="$6"
 
 libpath="-L${libdir}"
 
-# So that we keep -R options where required
-if test -n "$hardcode_libdir_flag_spec"; then
-  eval flag=\"$hardcode_libdir_flag_spec\"
-  libpath="$libpath $flag"
-fi
-
 cat <<EOF
 #!/bin/sh
 
-usage="usage: rep-config [--version] [--libs] [--cflags] [--execdir]"
+usage="usage: rep-config [--version] [--libs] [--cflags] [--execdir] [--lispdir]"
 
 if test \$# -eq 0; then
       echo "\${usage}" 1>&2
@@ -44,6 +39,9 @@ while test \$# -gt 0; do
     --execdir)
       echo "${repexecdir}"
       ;;
+    --lispdir)
+      echo "${sitelispdir}"
+      ;;
     *)
       echo "\${usage}" 1>&2
       exit 1
