@@ -1,5 +1,5 @@
 /* repsdbm.c -- rep wrapper to libsdbm
-   $Id: repsdbm.c,v 1.13 2000/09/03 12:36:04 john Exp $ */
+   $Id$ */
 
 #define _GNU_SOURCE
 
@@ -271,3 +271,14 @@ rep_dl_init (void)
     rep_ADD_SUBR(Ssdbmp);
     return rep_pop_structure (tem);
 }
+
+void
+rep_dl_kill (void)
+{
+    rep_dbm *db;
+    for (db = dbm_chain; db != 0; db = db->next)
+    {
+	if (db->dbm != 0)
+	    Fsdbm_close (rep_VAL (db));
+    }
+}
